* Knight Rider Light Revisited
* J. Grand 2/17/97
*
* Port A for LEDs (use pull-ups)
* Port B 0,1 used for light pattern selection:
*      0     Knight Rider
*      1     Police Wig-Wag
*      2     Star Trek: Inside to outside
*      3     Alternation light pattern
* Optional Job 4 Star Trek: Outside to inside
*
* Timing based on instruction execution times
* If using a simulator or crystal less than 4MHz, routine will run slower

$nolist
$include 'k1equ.asm'    ;EQUates for 68HC705K1 part
$list

$BASE   10T             ;Used for iASM (commented out)

TEMP1   EQU     $E0     ;One byte temp storage location

        ORG     $0200   ;Program will start at $0200

INIT    LDA     #$FF    ;Begin initialization
        STA     PORTA   ;All LEDs off
        STA     DDRA    ;Set Port A as outputs
        LDA     #$00
        STA     PORTB
        STA     DDRB    ;Set Port B as inputs

SELECT  LDA     PORTB   ;Check selection switch
        CMP     #0T     ;If 0
        BEQ     JOB0    ;Go to job 0
        CMP     #1T     ;If 1
        BEQ     JOB1    ;Go to job 1
        CMP     #2T     ;If 2
        BEQ     JOB2    ;Go to job 2
        JMP     JOB3    ;Else, must be job 3

JOB2                    ;Star Trek - inside to outside
        LDA     #$FF
        STA     PORTA   ;Turn off all LEDs
        BCLR    3,PORTA
        BCLR    4,PORTA ;Turn on two middle LEDs
        JSR     DLY50   ;Delay
        JSR     DLY50
        BCLR    2,PORTA
        BCLR    5,PORTA
        BSET    3,PORTA
        BSET    4,PORTA
        JSR     DLY50
        JSR     DLY50
        BCLR    1,PORTA
        BCLR    6,PORTA
        BSET    2,PORTA
        BSET    5,PORTA
        JSR     DLY50
        JSR     DLY50
        BCLR    0,PORTA
        BCLR    7,PORTA
        BSET    1,PORTA
        BSET    6,PORTA
        JSR     DLY50   ;Wait a bit at end
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JMP     SELECT  ;Check selection and repeat

JOB1                    ;Police "Wig-Wag"
        LDA     #$0F    ;Turn on left side of LED array
        STA     PORTA
        JSR     DLY50   ;Wait a bit
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        LDA     #$F0    ;Turn on right side of array
        STA     PORTA
        JSR     DLY50   ;Wait a bit more and repeat
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JMP     SELECT

JOB0                    ;Standard Knight Rider light-chaser
                        ;Could also use LSR and LSL PORTA
        BSET    1,PORTA ;Turn off bit 1
        BCLR    0,PORTA ;Turn on bit 0
        JSR     DLY50   ;Delay
        BCLR    1,PORTA ;Turn on bit 1
        BSET    0,PORTA ;Turn off bit 0
        JSR     DLY50   ;Delay
        BCLR    2,PORTA ;Turn on bit 2
        BSET    1,PORTA ;Turn off bit 1
        JSR     DLY50   ;Delay
        BCLR    3,PORTA ;Turn on bit 3
        BSET    2,PORTA ;Turn off bit 2
        JSR     DLY50
        BCLR    4,PORTA
        BSET    3,PORTA
        JSR     DLY50
        BCLR    5,PORTA
        BSET    4,PORTA
        JSR     DLY50
        BCLR    6,PORTA
        BSET    5,PORTA
        JSR     DLY50
        BCLR    7,PORTA
        BSET    6,PORTA
        JSR     DLY50   ;Go back the other direction
        BCLR    6,PORTA
        BSET    7,PORTA
        JSR     DLY50
        BCLR    5,PORTA
        BSET    6,PORTA
        JSR     DLY50
        BCLR    4,PORTA
        BSET    5,PORTA
        JSR     DLY50
        BCLR    3,PORTA
        BSET    4,PORTA
        JSR     DLY50
        BCLR    2,PORTA
        BSET    3,PORTA
        JSR     DLY50
        BCLR    1,PORTA
        BSET    2,PORTA
        JSR     DLY50
        BCLR    0,PORTA
        BSET    1,PORTA
        JSR     DLY50
        JSR     DLY50
        JMP     SELECT  ;Check selection and repeat

JOB4                    ;optional job Star Trek - outside to inside
        LDA     #$FF
        STA     PORTA   ;Turn off all LED's
        BCLR    0,PORTA
        BCLR    7,PORTA ;Turn on outside LED's
        JSR     DLY50
        BCLR    1,PORTA
        BCLR    6,PORTA
        BSET    0,PORTA
        BSET    7,PORTA
        JSR     DLY50
        BCLR    2,PORTA
        BCLR    5,PORTA
        BSET    1,PORTA
        BSET    6,PORTA
        JSR     DLY50
        BCLR    3,PORTA
        BCLR    4,PORTA
        BSET    2,PORTA
        BSET    5,PORTA
        JSR     DLY50   ;Wait a bit at the end
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JMP     SELECT  ;Check selection bits and repeat

JOB3                    ;Alternating light pattern
        LDA     #%01010101
        STA     PORTA
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        LDA     #%10101010
        STA     PORTA
        JSR     DLY50
        JSR     DLY50
        JSR     DLY50
        JMP     SELECT

* DLY50 - Subroutine to delay ~92mS
* Save original accumulator value
* but X will always be zero on return

DLY50
        STA     TEMP1   ;Save accumulator in RAM
        LDA     #120T   ;Do outer loop 120 times
OUTLP   CLRX            ;X used as inner loop count
INNRLP  DECX            ;0-FF, FF-FE,...1-0 256 loops
        BNE     INNRLP  ;6cyc*256*1S/cyc = 1.536mS
        DECA            ;120-119, 119-118,...1-0
        BNE     OUTLP   ;1545cyc*120*500ns/cyc=92.69mS
        LDA     TEMP1   ;Recover saved Accumulator val
        RTS             ;Return

        ORG     $03FE   ;Vector area
        FDB     INIT    ;Reset vector for ICS05
